"use strict";
;
;
;
;
;
;
;
var Point;
(function (Point) {
    var floor = Math.floor;
    function scale(scale, p) {
        return {
            x: floor(p.x * scale.x),
            y: floor(p.y * scale.y)
        };
    }
    Point.scale = scale;
    function scaleConstant(scale, p) {
        return {
            x: floor(p.x * scale),
            y: floor(p.y * scale)
        };
    }
    Point.scaleConstant = scaleConstant;
    function scaleAxis(axis, scale, p) {
        var ret = copy(p);
        ret[axis] = floor(p[axis] * scale);
        return ret;
    }
    Point.scaleAxis = scaleAxis;
    function copy(p) {
        return { x: p.x, y: p.y };
    }
    Point.copy = copy;
    function add(a, b) {
        return {
            x: a.x + b.x,
            y: a.y + b.y
        };
    }
    Point.add = add;
    function subtract(a, b) {
        return {
            x: a.x - b.x,
            y: a.y - b.y
        };
    }
    Point.subtract = subtract;
    Point.ZERO = { x: 0, y: 0 };
    function isPositive(p) {
        return p.x > 0 && p.y > 0;
    }
    Point.isPositive = isPositive;
    function eqTo(p, x, y) {
        return p.x === x && p.y === y;
    }
    Point.eqTo = eqTo;
    function ofEvent(event) {
        var _a = event.get_coords(), absx = _a[0], absy = _a[1];
        return { x: absx, y: absy };
    }
    Point.ofEvent = ofEvent;
    function magnitude(p) {
        return Math.sqrt(Math.pow(p.x, 2) + Math.pow(p.y, 2));
    }
    Point.magnitude = magnitude;
})(Point || (Point = {}));
var MathUtil;
(function (MathUtil) {
    function between(min, x, max) {
        // p('between('+min+', '+x+', '+max+')');
        if (min > x)
            return min;
        if (max < x)
            return max;
        return x;
    }
    MathUtil.between = between;
})(MathUtil || (MathUtil = {}));
/// <reference path="common.ts" />
/// <reference path="point.ts" />
/// <reference path="math.ts" />
var Rect;
(function (Rect) {
    function copy(r) {
        return { pos: Point.copy(r.pos), size: Point.copy(r.size) };
    }
    Rect.copy = copy;
    function area(r) {
        return r.size.x * r.size.y;
    }
    Rect.area = area;
    function isPositive(r) {
        return Point.isPositive(r.size);
    }
    Rect.isPositive = isPositive;
    function closestCorner(r, p) {
        var pow = Math.pow;
        var sqrt = Math.sqrt;
        var near = r.pos;
        var far = { x: r.pos.x + r.size.x, y: r.pos.y + r.size.y };
        var tl = sqrt(pow(p.x - near.x, 2) + pow(p.y - near.y, 2));
        var tr = sqrt(pow(p.x - far.x, 2) + pow(p.y - near.y, 2));
        var br = sqrt(pow(p.x - far.x, 2) + pow(p.y - far.y, 2));
        var bl = sqrt(pow(p.x - near.x, 2) + pow(p.y - far.y, 2));
        var corners = [tl, tr, bl, br];
        // log("Closest corners: " + JSON.stringify(corners));
        corners.sort(function (a, b) { return a - b; });
        // log("Closest corners (sorted): " + JSON.stringify(corners));
        var min = corners[0];
        if (min == tl)
            return 1 /* TOPLEFT */;
        if (min == tr)
            return 3 /* TOPRIGHT */;
        if (min == br)
            return 5 /* BOTTOMRIGHT */;
        return 7 /* BOTTOMLEFT */;
    }
    Rect.closestCorner = closestCorner;
    function move(r, p) {
        return {
            pos: Point.add(r.pos, p),
            size: r.size
        };
    }
    Rect.move = move;
    function moveWithin(rect, bounds) {
        var r = copy(rect);
        if (r.size.x > bounds.size.x)
            r.size.x = bounds.size.x;
        if (r.size.y > bounds.size.y)
            r.size.y = bounds.size.y;
        r.pos.x = MathUtil.between(bounds.pos.x, r.pos.x, (bounds.pos.x + bounds.size.x - r.size.x));
        r.pos.y = MathUtil.between(bounds.pos.y, r.pos.y, (bounds.pos.y + bounds.size.y - r.size.y));
        return r;
    }
    Rect.moveWithin = moveWithin;
    function scaleWithin(rect, size) {
        return moveWithin(rect, { pos: Point.ZERO, size: size });
    }
    Rect.scaleWithin = scaleWithin;
    function midpoint(rect) {
        return Point.add(rect.pos, Point.scaleConstant(0.5, rect.size));
    }
    Rect.midpoint = midpoint;
    function moveWithinAffinity(opts) {
        var abs = Math.abs;
        var original = opts.original, bounds = opts.bounds;
        var modified = scaleWithin(opts.modified, bounds);
        function close(a, b) {
            return abs(a - b) < opts.margin;
        }
        function axisDiff(axis) {
            var near = close(0, original.pos[axis]);
            var far = close((bounds[axis]), (original.pos[axis] + original.size[axis]));
            // p("near affinity["+axis+"] = "+near);
            // p("far affinity["+axis+"] = "+far);
            if (near && far)
                return 0; // can't do anything for a window attached to both edges
            if (near) {
                // p("maintaining near affinity");
                return -modified.pos[axis];
            }
            else if (far) {
                // p("maintaining far affinity");
                return (bounds[axis]) - (modified.pos[axis] + modified.size[axis]);
            }
            return 0;
        }
        return move(modified, {
            x: axisDiff("x" /* x */),
            y: axisDiff("y" /* y */)
        });
    }
    Rect.moveWithinAffinity = moveWithinAffinity;
    function intersection(a, b) {
        var pos = {
            x: Math.max(a.pos.x, b.pos.x),
            y: Math.max(a.pos.y, b.pos.y)
        };
        var bottomRightA = Point.add(a.pos, a.size);
        var bottomRightB = Point.add(b.pos, b.size);
        var bottomRight = {
            x: Math.min(bottomRightA.x, bottomRightB.x),
            y: Math.min(bottomRightA.y, bottomRightB.y)
        };
        var size = Point.subtract(bottomRight, pos);
        if (Point.isPositive(size)) {
            return { pos: pos, size: size };
        }
        else {
            return null;
        }
    }
    Rect.intersection = intersection;
})(Rect || (Rect = {}));
/// <reference path="rect.ts" />
var assert = require('assert');
describe('Rect', function () {
    it('intersects', function () {
        assert.deepEqual({
            pos: { x: 50, y: 25 },
            size: { x: 200, y: 100 }
        }, Rect.intersection({
            pos: { x: 50, y: 0 },
            size: { x: 300, y: 125 }
        }, {
            pos: { x: 0, y: 25 },
            size: { x: 250, y: 300 }
        }));
    });
});
